# rubocop:disable Lint/MissingCopEnableDirective, Metrics/LineLength

require 'etc'
require 'tempfile'
require 'thor'
require 'magma/preparer'
require 'magma/reifier'
require 'magma/renderer'
require 'magma/templater'
require 'yaml'

module Magma
  CONFIG_FILE = File.join(Dir.home, '.magmarc')
  CONFIG = (File.exist?(CONFIG_FILE) ? (YAML.load_file(CONFIG_FILE) || {}) : {})

  # The CLI application
  class App < Thor
    class << self
      def exit_on_failure?
        true
      end
    end

    desc 'config', 'Get or sets global configuration'
    option :backend, type: :string, desc: 'Backend, e.g. melt or qmelt'
    def config
      config = CONFIG.merge(options)
      File.open(CONFIG_FILE, 'w') { |f| f.write config.to_yaml }
      puts config.to_yaml
    end

    desc 'reify [manifest]', 'Reifies supporting assets'
    def reify(data_file)
      Reifier.call data_file, options
    end

    desc 'render [outfile]', 'Renders an MLT into an HLS video'
    option :context, type: :hash, default: {}, aliases: 'c', desc: 'Render-time context'
    option :data_file, type: :string, aliases: 'D', desc: 'Static data file, accepts JSON or YML'
    option :enable_real_time, type: :boolean, default: false, aliases: 'r', desc: 'Render in real-time'
    option :format, type: :string, default: 'mp4', aliases: 'f', enum: Renderer::SUPPORTED_FORMATS, desc: 'Encoding format'
    option :globals, type: :hash, default: {}, aliases: 'g', desc: 'Global data overrides'
    option :infile, type: :string, aliases: 'i', desc: 'Input file', required: true
    option :length, type: :numeric, desc: 'Set the target length in seconds'
    option :preset, type: :string, default: 'fast', desc: 'Video preset for encoding'
    option :real_time, type: :numeric, default: Etc.nprocessors, aliases: 'n', desc: 'Number of cores (real-time)'
    option :size, type: :string, aliases: 's', desc: 'Frame size (width x height)'
    option :skip_template, type: :boolean, default: false, desc: 'Skips template-filling step'
    option :vcodec, type: :string, desc: 'Video codec to use for encoding'
    def render(outfile)
      if options[:skip_template]
        Renderer.call outfile, options
      else
        template = Tempfile.new [File.basename(outfile), '.mlt']

        begin
          Reifier.call(options[:data_file]) if options[:data_file]
          Preparer.call options[:infile], options.merge(outfile: template)
          Templater.call template, options.merge(outfile: template)
          Renderer.call outfile, options.merge(infile: template.path)
        ensure
          template.close
          template.unlink
        end
      end
    end

    desc 'template [file]', 'Fills a template'
    option :context, type: :hash, default: {}, aliases: 'c', desc: 'Render-time context'
    option :globals, type: :hash, default: {}, aliases: 'g', desc: 'Global data overrides'
    option :data_file, type: :string, aliases: 'D', desc: 'Static data file, accepts JSON or YML'
    option :outfile, type: :string, aliases: 'o', desc: 'Output file'
    option :overwrite, type: :boolean, default: false, aliases: 'O', desc: 'Overwrite input file'
    option :print, type: :string, aliases: 'p', desc: 'Print output'
    def template(infile = nil)
      Templater.call infile, options
    end

    desc 'prepare [file]', 'Prepares a template'
    option :outfile, type: :string, aliases: 'o', desc: 'Output file'
    option :overwrite, type: :boolean, default: false, aliases: 'O', desc: 'Overwrite input file'
    option :print, type: :string, aliases: 'p', desc: 'Print output'
    def prepare(infile = nil)
      Preparer.call infile, options
    end

    desc 'version', 'Displays the version'
    def version
      puts Magma::VERSION
    end
  end
end
