require 'magma/utils'

module Magma
  class Renderer
    SUPPORTED_FORMATS = %w[hls mp4].freeze

    include Magma::Common
    include Magma::Utils

    def initialize(outfile, options)
      @outfile = outfile
      @options = options
    end

    def call
      render!
    end

    private

    attr_accessor :options, :outfile

    # Adds sane defaults to config
    def add_defaults(options)
      {
        hls_list_size: 0,
        start_number: 0,
      }.merge(options)
    end

    # Gets the melt backend
    def backend
      Magma::CONFIG['backend'] || 'melt'
    end

    # Formats keys to melt arguments
    def format(options)
      options.map do |key, value|
        [{
          format: :f,
          size: :s,
          length: :hls_time,
        }[key] || key, value].map(&:to_s).join('=')
      end
    end

    # Finds and constructs a melt invocation string
    def melt(args)
      bin = which(backend) || raise("#{backend} not found. Install MLT or set a backend (magma config)")
      "#{bin} #{args}"
    end

    # Constructs melt arguments from options hash
    def melt_args(options)
      [
        options[:infile],
        "-consumer avformat:#{outfile}",
      ].concat(pipe(options, %i[
        symbolize_keys
        add_defaults
        select
        transform
        table
        format
      ]))
    end

    # Renders the project
    def render!
      cmd = melt melt_args(options.merge(outfile: outfile)).join(' ')
      puts "Run: #{cmd}"
      puts

      run(cmd) do |_stdin, _stdout, stderr|
        stderr.each("\r") do |line|
          STDOUT.write "\r#{line}"
        end
      end
    end

    # Selects certain options
    def select(options)
      # Clone original
      options = options.clone

      # Handle related options
      options.delete(:real_time) unless options.delete(:enable_real_time)

      # Reject
      options.select do |key, value|
        !value.nil? && %i[
          format
          hls_list_size
          real_time
          length
          preset
          size
          start_number
          vcodec
        ].include?(key)
      end
    end

    # Prints a table and passes through the options hash
    def table(options)
      lpadding = options.keys.max_by(&:length).length + 2
      rpadding = options.values.max_by { |v| v.to_s.length }.to_s.length
      puts 'PROPERTY'.ljust(lpadding) + 'VALUE'
      puts '=' * (lpadding + rpadding)
      options.keys.sort.each do |key|
        puts key.to_s.ljust(lpadding) + options[key].to_s
      end
      puts
      options
    end

    # Transforms certain options values
    def transform(options)
      options.map do |key, value|
        [key, ({
          real_time: ->(x) { "-#{x}" },
        }[key] || proc { |x| x }).call(value)]
      end.to_h
    end
  end
end
