require 'open3'

module Magma
  module Utils
    # Reads a file or from STDIN if piped
    def read_file_or_stdin(filename = nil)
      filename.nil? ? !STDIN.tty? && STDIN.read : File.read(filename)
    end

    # Returns whether a string is a UUID
    def uuid?(uuid)
      uuid_regex = /^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/
      uuid_regex =~ uuid.to_s.downcase
    end

    # Applies methods/procs to an object in order
    def pipe(object, methods = [])
      methods.reduce(object) do |acc, method|
        case method
        when Proc || method.respond_to?(:call)
          method.call(acc)
        when Symbol
          send(method, acc)
        when String
          raise "must start with '.'" unless method[0] == '.'
          acc.send(method[1..-1].to_sym)
        else
          raise "unexpected pipe #{method}"
        end
      end
    end

    # Delegates to Open3
    def run(*args, &block)
      return Open3.popen3(*args, &block) if block_given?
      Open3.popen3(*args)
    end

    # Symbolizes a hash's keys
    def symbolize_keys(hash)
      hash.each_with_object({}) { |(k, v), memo| memo[k.to_sym] = v }
    end

    # Cross-platform way of finding an executable in the $PATH
    # Reference: https://stackoverflow.com/a/5471032/3557448
    def which(cmd)
      return cmd if File.exist?(cmd) && File.executable?(cmd)
      exts = ENV['PATHEXT'] ? ENV['PATHEXT'].split(';') : ['']
      ENV['PATH'].split(File::PATH_SEPARATOR).each do |path|
        exts.each do |ext|
          exe = File.join(path, "#{cmd}#{ext}")
          return exe if File.executable?(exe) && !File.directory?(exe)
        end
      end
      nil
    end
  end
end
